@@ -322,6 +322,7 @@ Point Basic_buffer::eot( const Point & p ) const throw()
   return p1;
   }
 
+
 // begin of paragraph
 Point Basic_buffer::bop( const Point & p ) const throw()
   {
@@ -330,6 +331,7 @@ Point Basic_buffer::bop( const Point & p ) const throw()
   return bol( p1 );
   }
 
+
 // end of paragraph
 Point Basic_buffer::eop( const Point & p ) const throw()
   {
@@ -342,12 +344,13 @@ Point Basic_buffer::eop( const Point & p ) const throw()
 // Move 'p' to the matching delimiter.
 // If (force && forward), fail if [p] is closing delimiter.
 // If (force && !forward), fail if [p] is opening delimiter.
+// Returns +1 if done, 0 if [p] is not a delimiter, -1 if no matching delimiter.
 //
-bool Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
-                                              const bool force ) const throw()
+int Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
+                                             const bool force ) const throw()
   {
   int ch1 = (*this)[p];
-  if( ch1 < 0 ) return false;
+  if( ch1 < 0 ) return 0;
   unsigned char ch2;
   const bool forward_orig = forward;
   bool comment = false;
@@ -366,29 +369,30 @@ bool Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
     case '"':
     case '\'':
     case '`': ch2 = ch1; break;
-    default: return false;
+    default: return 0;
     }
-  if( force && ( forward != forward_orig ) ) return false;
+  if( force && ( forward != forward_orig ) ) return -1;
 
   if( comment )
     {
     Point p1 = p;
-    int ch = pngetc( p1 );
+    const int ch = pngetc( p1 );
     if( ch >= 0 && ch == ch2 )
       {
       if( forward )
-        { if( find_text( p1, std::string( "*/" ) ) )
-          { pseek( p1, -2 ); p = p1; return true; } }
+        { if( pngetc( p1 ) >= 0 && find_text( p1, std::string( "*/" ) ) )
+          { pseek( p1, -2 ); p = p1; return +1; } }
       else
-        { if( rfind_text( p1, std::string( "/*" ) ) )
-          { p = p1; return true; } }
+        { p1 = p; if( rfind_text( p1, std::string( "/*" ) ) )
+          { p = p1; return +1; } }
+      return -1;
       }
-    return false;
+    return 0;
     }
 
   { Point pt = p;		// no match if delimiter is escaped
     if( ppgetc( pt ) == '\\' )
-      { Point pt2 = pt; if( ppgetc( pt2 ) != '\\' ) return false; } }
+      { Point pt2 = pt; if( ppgetc( pt2 ) != '\\' ) return -1; } }
   Point p1 = p;
   int level = 0;	// nesting level of delimiters
   bool quote = false;	// ignore double-quoted text
@@ -399,7 +403,7 @@ bool Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
       const int ch = pngetc( p1 ); if( ch < 0 ) break;
       if( !quote )
         {
-        if( ch == ch2 ) { if( --level < 0 ) { p = p1; return true; } }
+        if( ch == ch2 ) { if( --level < 0 ) { p = p1; return +1; } }
         else if( ch == ch1 ) { ++level; continue; }
         else if( ch == '\'' )		// ignore single-quoted characters
           {
@@ -421,7 +425,7 @@ bool Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
         Point pt = p1;
         if( ppgetc( pt ) == '\\' )
           { Point pt2 = pt; if( ppgetc( pt2 ) != '\\' ) { p1 = pt; continue; } }
-        if( ch == ch2 ) { if( --level < 0 ) { p = p1; return true; } }
+        if( ch == ch2 ) { if( --level < 0 ) { p = p1; return +1; } }
         else if( ch == ch1 ) { ++level; continue; }
         else if( ch == '\'' )		// ignore single-quoted characters
           {
@@ -439,7 +443,7 @@ bool Basic_buffer::set_to_matching_delimiter( Point & p, bool forward,
         quote = !quote;
         }
       }
-  return false;
+  return -1;
   }
 
 
