@@ -24,6 +24,7 @@
 #include "buffer.h"
 #include "block.h"
 #include "rc.h"
+#include "screen.h"
 
 
 namespace Block {
@@ -42,10 +43,7 @@ bool marking() throw() { return ( _anchor != Point() ); }
 } // end namespace Block
 
 
-Buffer & Block::buffer() throw() { return *_bufferp; }
-
-bool Block::same_buffer( const Buffer & buffer ) throw()
-  { return &buffer == _bufferp; }
+Buffer * Block::bufferp() throw() { return _bufferp; }
 
 Point & Block::anchor() throw() { return _anchor; }
 
@@ -55,18 +53,27 @@ Point & Block::end() throw() { return _end; }
 
 
 bool Block::in_block( const Buffer & buffer, const Point & p ) throw()
-  { return ( &buffer == _bufferp && _begin.line >= 0 && p >= _begin && p < _end ); }
+  {
+  return ( &buffer == _bufferp && _begin.line >= 0 && p >= _begin && p < _end &&
+           ( !RC::editor_options().rectangle_mode ||
+             ( p.col >= _begin.col && p.col < _end.col ) ) );
+  }
 
 
 bool Block::in_block_or_end( const Buffer & buffer, const Point & p ) throw()
   {
   return ( &buffer == _bufferp && _begin.line >= 0 && _begin < _end &&
-           p >= _begin && p <= _end );
+           p >= _begin && p <= _end &&
+           ( !RC::editor_options().rectangle_mode ||
+             ( p.col >= _begin.col && p.col <= _end.col ) ) );
   }
 
 
 bool Block::valid() throw()
-  { return ( _bufferp != 0 && _begin.line >= 0 && _begin < _end ); }
+  {
+  return ( _bufferp != 0 && _begin.line >= 0 && _begin < _end &&
+           ( !RC::editor_options().rectangle_mode || _begin.col < _end.col ) );
+  }
 
 
 void Block::save_block_position() throw()
@@ -107,7 +114,7 @@ void Block::set_end( Buffer & buffer, const Point & p ) throw()
 bool Block::follow_marking( const Buffer & buffer, const Point & p ) throw()
   {
   static Point old_p;
-  if( marking() && same_buffer( buffer ) && p != old_p )
+  if( marking() && &buffer == _bufferp && p != old_p )
     {
     if( p < _anchor ) _begin = p; else _begin = _anchor;
     if( p > _anchor ) _end = p; else _end = _anchor;
@@ -118,13 +125,14 @@ bool Block::follow_marking( const Buffer & buffer, const Point & p ) throw()
   }
 
 
-const char * Block::toggle_marking( Buffer & buffer, const Point & p ) throw()
+void Block::toggle_marking( Buffer & buffer, const Point & p ) throw()
   {
   const char * const str[2] = { "Selection started.", "Selection cleared." };
   const char * msg = 0;
+  const Buffer * old_bufferp = _bufferp;
   if( marking() )
     {
-    if( same_buffer( buffer ) ) { if( valid() ) disable_marking(); else reset(); }
+    if( &buffer == _bufferp ) { if( valid() ) disable_marking(); else reset(); }
     else { enable_marking( buffer, p ); msg = str[0]; }
     }
   else
@@ -132,57 +140,96 @@ const char * Block::toggle_marking( Buffer & buffer, const Point & p ) throw()
     if( in_block_or_end( buffer, p ) ) { reset(); msg = str[1]; }
     else { enable_marking( buffer, p ); msg = str[0]; }
     }
-  return msg;
+  if( msg )
+    {
+    if( old_bufferp ) Screen::repaint( old_bufferp );
+    Screen::show_message( msg );
+    }
   }
 
 
-bool Block::copy_block( Buffer & buffer, const Point & p ) throw()
+bool Block::copy_block( Buffer & buffer, const Point & p, const bool move ) throw()
   {
+  bool done = false;
   if( buffer.options.read_only || !buffer.pisvalid( p ) || !valid() ||
+      ( move && _bufferp->options.read_only ) ||
       !_bufferp->pisvalid( _begin ) || !_bufferp->pisvalid( _end ) )
-    return false;
-  Basic_buffer tmp( *_bufferp, _begin, _end );
-  Point p2 = p;
+    return done;
+  Point p_end;
+  const bool same = ( &buffer == _bufferp );
   buffer.reset_appendable();
-  bool done = buffer.pputb( p2, tmp, tmp.bof(), tmp.eof() );
+  if( !RC::editor_options().rectangle_mode )
+    {
+    Basic_buffer tmp( *_bufferp, _begin, _end );
+    if( move ) { delete_block(); if( same ) buffer.force_append(); }
+    p_end = p;
+    done = buffer.pputb( p_end, tmp, tmp.bof(), tmp.eof() );
+    }
+  else
+    {
+    Basic_buffer tmp;
+    Point p0 = tmp.bof();
+    const int len = _end.col - _begin.col;
+    for( int line = _begin.line; line <= _end.line; ++line )
+      {
+      Point p1( line, _begin.col ), p2( line, _end.col );
+      _bufferp->pvalid( p1 ); _bufferp->pvalid( p2 );
+      tmp.pputb( p0, *_bufferp, p1, p2 );
+      for( int i = tmp.characters( p0.line ); i < len; ++i )
+        tmp.pputc( p0, ' ' );
+      tmp.pputc( p0, '\n' );
+      }
+    if( move ) { delete_block(); if( same ) buffer.force_append(); }
+    for( int line = 0; line <= tmp.last_line(); ++line )
+      {
+      p0 = p; p0.line += line;
+      for( int i = buffer.lines(); i <= p0.line; ++i )
+        {
+        Point dummy = buffer.eof();
+        const bool ai = buffer.options.auto_indent;
+        buffer.options.auto_indent = false;
+        buffer.pputc( dummy, '\n' ); buffer.force_append();
+        buffer.options.auto_indent = ai;
+        }
+      if( line >= tmp.last_line() ) break;
+      for( int i = buffer.eol( p0 ).col; i < p0.col; ++i )
+        { Point dummy = buffer.eol( p0 );
+          buffer.pputc( dummy, ' ' ); buffer.force_append(); }
+      Point p1( line, 0 ), p2 = tmp.eol( p1 );
+      if( buffer.pputb( p0, tmp, p1, p2 ) ) { done = true; p_end = p0; }
+      buffer.force_append();
+      }
+    }
   buffer.reset_appendable();
   if( done )
-    { if( RC::editor_options().auto_unmark ) reset(); else set_block( buffer, p, p2 ); }
+    { if( RC::editor_options().auto_unmark ) reset();
+      else set_block( buffer, p, p_end ); }
   return done;
   }
 
 
 bool Block::delete_block() throw()
   {
+  bool done = false;
   if( valid() )
     {
-    Buffer & buffer = *_bufferp;
+    Buffer * const bp = _bufferp;
     _bufferp = 0;			// disable adjust functions on block
-    buffer.reset_appendable();
-    const bool result = buffer.pdelb( _begin, _end );
-    buffer.reset_appendable();
-    _bufferp = &buffer; _end = _begin; disable_marking();
-    return result;
+    bp->reset_appendable();
+    if( !RC::editor_options().rectangle_mode )
+      done = bp->pdelb( _begin, _end );
+    else
+      {
+      for( int line = _end.line; line >= _begin.line; --line )
+        {
+        Point p1( line, _begin.col ), p2( line, _end.col );
+        bp->pvalid( p1 ); bp->pvalid( p2 );
+        if( bp->pdelb( p1, p2 ) ) done = true;
+        bp->force_append();
+        }
+      }
+    bp->reset_appendable();
+    _bufferp = bp; _end = _begin; disable_marking();
     }
-  return false;
-  }
-
-
-bool Block::move_block( Buffer & buffer, const Point & p ) throw()
-  {
-  if( buffer.options.read_only || !buffer.pisvalid( p ) ||
-      !valid() || _bufferp->options.read_only ||
-      !_bufferp->pisvalid( _begin ) || !_bufferp->pisvalid( _end ) )
-    return false;
-  Basic_buffer tmp( *_bufferp, _begin, _end );
-  const bool same = same_buffer( buffer );
-  delete_block();
-  if( same ) buffer.force_append();
-  else buffer.reset_appendable();
-  Point p2 = p;
-  bool done = buffer.pputb( p2, tmp, tmp.bof(), tmp.eof() );
-  buffer.reset_appendable();
-  if( done )
-    { if( RC::editor_options().auto_unmark ) reset(); else set_block( buffer, p, p2 ); }
   return done;
   }
