@@ -18,8 +18,7 @@
 
 namespace Block {
 
-Buffer & buffer() throw();
-bool same_buffer( const Buffer & buffer ) throw();
+Buffer * bufferp() throw();
 Point & anchor() throw();		// anchor for marking
 Point & begin() throw();
 Point & end() throw();
@@ -36,10 +35,9 @@ void set_begin( Buffer & buffer, const Point & p ) throw();
 void set_end( Buffer & buffer, const Point & p ) throw();
 
 bool follow_marking( const Buffer & buffer, const Point & p ) throw();
-const char * toggle_marking( Buffer & buffer, const Point & p ) throw();
+void toggle_marking( Buffer & buffer, const Point & p ) throw();
 
-bool copy_block( Buffer & buffer, const Point & p ) throw();
+bool copy_block( Buffer & buffer, const Point & p, const bool move = false ) throw();
 bool delete_block() throw();
-bool move_block( Buffer & buffer, const Point & p ) throw();
 
 } // end namespace Block
