@@ -55,7 +55,7 @@ std::vector< Point * > & get_marks_and_pointers( Buffer & b ) throw()
     Buffer_handle & bh = bufhandle_vector[i];
     if( &b == &bh.buffer() ) v.push_back( &bh.pointer );
     }
-  if( Block::same_buffer( b ) )
+  if( Block::bufferp() == &b )
     {
     v.push_back( &Block::anchor() );
     v.push_back( &Block::begin() );
@@ -182,8 +182,8 @@ Buffer::Buffer( const Options & opts, const std::string * namep ) throw( Buffer:
     }
   if( buf ) std::free( buf );
 
-  bool file_error = std::ferror( f );
-  bool file_eof = std::feof( f );
+  const bool file_error = std::ferror( f );
+  const bool file_eof = std::feof( f );
   if( f != stdin )
     {
     if( std::freopen( _name.c_str(), "r+", f ) ) std::fclose( f );
@@ -230,15 +230,19 @@ bool Buffer::save( const std::string * namep, bool set_name, bool append,
     }
   if( !namep ) return false;
 
-  if( same_name && !saved && RC::editor_options().backup )
+  if( same_name )
     {
-    std::string backname = *namep; backname += '~';
-    std::remove( backname.c_str() );
-    std::rename( namep->c_str(), backname.c_str() );
+    if( options.read_only ) return false;
+    if( !saved && RC::editor_options().backup )
+      {
+      std::string backname = *namep; backname += '~';
+      std::remove( backname.c_str() );
+      std::rename( namep->c_str(), backname.c_str() );
+      }
     }
 
   FILE *f = 0;
-  if( Menu_file::make_path( *namep ) )
+  if( Menu::make_path( *namep ) )
     f = std::fopen( namep->c_str(), append ? "a" : "w" );
   if( !f )
     { if( !had_name ) { _name.clear(); } throw Error( "Error opening file" ); }
@@ -274,18 +278,18 @@ const std::string & Buffer::character_info( const Point & p ) const throw()
   int ch = (*this)[p];
 
   if( ch >= 0 )
-    std::snprintf( tmp2, sizeof( tmp2 ), "Code %d(0x%X/0%o) **", ch, ch ,ch );
+    snprintf( tmp2, sizeof( tmp2 ), "Code %d(0x%X/0%o) **", ch, ch ,ch );
   else if( p == eof() )
-    std::snprintf( tmp2, sizeof( tmp2 ), "(End Of File) **" );
+    snprintf( tmp2, sizeof( tmp2 ), "(End Of File) **" );
   else
-    { std::snprintf( tmp2, sizeof( tmp2 ), "*** Error: Bad Pointer: Line %d  Col %d",
+    { snprintf( tmp2, sizeof( tmp2 ), "*** Error: Bad Pointer: Line %d  Col %d",
                 p.line + 1, p.col + 1 ); s = tmp2; return s; }
 
   int offset = p.col;
   for( int i = 0; i < p.line; ++i ) offset += characters( i );
   if( crlf ) { offset += p.line; if( ch == '\n' ) ++offset; }
 
-  std::snprintf( tmp1, sizeof( tmp1 ), "** Line %d  Col %d  Offset %d(0x%X)  ",
+  snprintf( tmp1, sizeof( tmp1 ), "** Line %d  Col %d  Offset %d(0x%X)  ",
             p.line + 1, p.col + 1, offset, offset );
   s = tmp1; s += tmp2;
   return s;
@@ -556,8 +560,9 @@ Bufhandle_vector::~Bufhandle_vector() throw()
   }
 
 
-bool Bufhandle_vector::save_all_named() const throw()
+void Bufhandle_vector::save_all_named() const throw()
   {
+  int count = 0;
   bool done = true;
 
   for( int i = 0; i < handles(); ++i )
@@ -565,11 +570,34 @@ bool Bufhandle_vector::save_all_named() const throw()
     const Buffer & buffer = data[i].buffer();
     if( buffer.modified() && buffer.name().size() )
       {
-      try { if( !buffer.save() ) done = false; }
+      try { if( buffer.save() ) ++count; else done = false; }
       catch( Buffer::Error ) { done = false; }
       }
     }
-  return done;
+  const char * s = ( done ? "" : "Only " );
+  char buf[80];
+  snprintf( buf, sizeof( buf ), "%s%d named buffer(s) saved", s, count );
+  Screen::show_status_lines();
+  Screen::show_message( buf );
+  }
+
+
+void Bufhandle_vector::show_status() const throw()
+  {
+  int mcount = 0, ucount = 0;
+
+  for( int i = 0; i < handles(); ++i )
+    {
+    if( data[i].buffer().modified() ) ++mcount;
+    if( !data[i].buffer().name().size() ) ++ucount;
+    }
+
+  char buf[80];
+  snprintf( buf, sizeof( buf ),
+            "**  %d modified and  %d unnamed out of  %d total buffer(s) **",
+            mcount, ucount, handles() );
+  Screen::show_status_lines();
+  Screen::show_message( buf );
   }
 
 
@@ -630,7 +658,7 @@ bool Bufhandle_vector::delete_handle( int i ) throw()
     if( j != i && &buffer == &data[j].buffer() ) break;
   if( j >= handles() )			// last handle for this buffer
     {
-    if( Block::same_buffer( buffer ) ) Block::reset();
+    if( Block::bufferp() == &buffer ) Block::reset();
     delete &buffer;
     }
   data.erase( data.begin() + i );
