@@ -67,7 +67,7 @@ protected:
   bool detect_and_remove_cr() throw();
 
 public:
-  Basic_buffer() : data( 1 ) {}
+  Basic_buffer() throw() : data( 1 ) {}
   Basic_buffer( const Basic_buffer & b, const Point & p1, const Point & p2 ) throw();
 
   bool blank( const int line ) const throw()
@@ -110,8 +110,8 @@ public:
   Point bop( const Point & p ) const throw();	// begin of paragraph
   Point eop( const Point & p ) const throw();	// end of paragraph
 
-  bool set_to_matching_delimiter( Point & p, bool forward = true,
-                                  const bool force = false ) const throw();
+  int set_to_matching_delimiter( Point & p, bool forward = true,
+                                 const bool force = false ) const throw();
   bool find_text( Point & p, const std::string & text, const bool icase = false ) const throw();
   bool rfind_text( Point & p, const std::string & text, const bool icase = false ) const throw();
 
@@ -230,7 +230,8 @@ public:
   bool undo( Point & p ) throw();
   bool redo( Point & p ) throw();
   void force_append() const throw() { change_history.force_append = true; }
-  void reset_appendable() const throw() { change_history.appendable = false; }
+  void reset_appendable() const throw()
+    { change_history.appendable = change_history.force_append = false; }
   };
 
 
@@ -263,7 +264,8 @@ class Bufhandle_vector
 public:
   Bufhandle_vector() throw() : _first( 0 ) {}
   ~Bufhandle_vector() throw();
-  bool save_all_named() const throw();
+  void save_all_named() const throw();
+  void show_status() const throw();
 
   int absolute_index( const int i ) const throw()
     { return ( i + _first ) % handles(); }
