@@ -168,15 +168,15 @@ bool isupper( const unsigned char ch ) throw()
 } // end namespace ISO_8859
 
 
-// 'seq' contains a escape sequence (the characters following a '\')
-// Returns the corresponding code and, in *lenp, the characters read
-// Returns -1 if error
+// 'seq[i]' begins a escape sequence (the characters following a '\').
+// Returns the corresponding code and, in *lenp, the characters read.
+// Returns -1 if error.
 //
-int ISO_8859::escape( const std::string & seq, int *lenp ) throw()
+int ISO_8859::escape( const std::string & seq, const unsigned int i, int *lenp ) throw()
   {
-  if( seq.size() == 0 ) return -1;
+  if( i >= seq.size() ) return -1;
   int len = 1;
-  unsigned char ch = seq[0];
+  unsigned char ch = seq[i];
 
   switch( ch )
     {
@@ -191,15 +191,15 @@ int ISO_8859::escape( const std::string & seq, int *lenp ) throw()
     case '0':
     case '1':
     case '2':
-    case '3': if( seq.size() < 3 || !ISO_8859::isodigit( seq[1] ) ||
-                  !ISO_8859::isodigit( seq[2] ) ) return -1;
-              ch = ((ch-'0') << 6) + ((seq[1]-'0') << 3) + (seq[2]-'0');
+    case '3': if( i + 2 >= seq.size() || !ISO_8859::isodigit( seq[i+1] ) ||
+                  !ISO_8859::isodigit( seq[i+2] ) ) return -1;
+              ch = ((ch-'0') << 6) + ((seq[i+1]-'0') << 3) + (seq[i+2]-'0');
               len = 3;
               break;
     case 'x':
-    case 'X': if( seq.size() < 3 || !std::isxdigit( seq[1] ) ||
-                  !std::isxdigit( seq[2] ) ) return -1;
-              ch = (ISO_8859::xtoi(seq[1]) << 4) + ISO_8859::xtoi(seq[2]);
+    case 'X': if( i + 2 >= seq.size() || ISO_8859::xtoi( seq[i+1] ) < 0 ||
+                  ISO_8859::xtoi( seq[i+2] ) < 0 ) return -1;
+              ch = (ISO_8859::xtoi( seq[i+1] ) << 4) + ISO_8859::xtoi( seq[i+2] );
               len = 3;
               break;
     }
@@ -208,38 +208,47 @@ int ISO_8859::escape( const std::string & seq, int *lenp ) throw()
   }
 
 
-// Replace the escape sequences for special characters; "\n" --> '\n'
-//
-void ISO_8859::deescapize( std::string & s ) throw()
+void ISO_8859::escapize( std::string & s ) throw()
   {
-  unsigned int i = 0;
-  while( true )
+  for( unsigned int i = 0; i < s.size(); ++i )
     {
-    while( i < s.size() && s[i] != '\\' ) ++i;
-    if( ++i >= s.size() ) break;
-    unsigned char ch = s[i];
-    switch( ch )
+    if( !ISO_8859::iscntrl( s[i] ) && s[i] != '\\' ) continue;
+    s.insert( i, 1, '\\' ); ++i;
+    switch( s[i] )
       {
-      case 'a': ch = '\a'; break;
-      case 'b': ch = '\b'; break;
-      case 'e': ch = 27; break;
-      case 'f': ch = '\f'; break;
-      case 'n': ch = '\n'; break;
-      case 'r': ch = '\r'; break;
-      case 't': ch = '\t'; break;
-      case 'v': ch = '\v'; break;
-      default: ch = 0;
+      case '\\': break;
+      case '\a': s[i] = 'a'; break;
+      case '\b': s[i] = 'b'; break;
+      case '\f': s[i] = 'f'; break;
+      case '\n': s[i] = 'n'; break;
+      case '\r': s[i] = 'r'; break;
+      case '\t': s[i] = 't'; break;
+      case '\v': s[i] = 'v'; break;
+      default:
+        {
+        s.insert( i, 1, 'x' ); ++i;
+        const unsigned char ch = s[i];
+        s.insert( i, 1, ISO_8859::xdigit( ch >> 4 ) ); ++i;
+        s[i] = ISO_8859::xdigit( ch & 0x0f );
+        }
       }
-    if( ch ) { s[i-1] = ch; s.erase( i, 1 ); }
-    else ++i;
     }
   }
 
 
-void ISO_8859::detabize( std::string & s ) throw()
+void ISO_8859::deescapize( std::string & s ) throw()
   {
-  for( int i = s.size(); --i >= 0; )
-    if( s[i] == '\t' ) { s[i] = 't'; s.insert( i, 1, '\\' ); }
+  for( unsigned int i = 0; i < s.size(); ++i )
+    {
+    if( s[i] != '\\' ) continue;
+    if( i + 1 >= s.size() ) break;
+    int len, c = ISO_8859::escape( s, i + 1, &len );
+    if( c < 0 ) continue;
+    const unsigned char ch = s[i+1];
+    if( ( ch != c && ch != '\\' ) || len > 1 )
+      { s[i] = c; s.erase( i + 1, len ); }
+    else ++i;
+    }
   }
 
 
@@ -255,14 +264,6 @@ bool ISO_8859::isalnum_( const unsigned char ch ) throw()
   }
 
 
-bool ISO_8859::isblank( const unsigned char ch ) throw()
-  { return ( ch == ' ' || ch == '\t' ); }
-
-
-bool ISO_8859::iscntrl( const unsigned char ch ) throw()
-  { return ( ch < 32 || ( ch >= 0x7f && ch < 0xA0 ) ); }
-
-
 bool ISO_8859::isodigit( const unsigned char ch ) throw()	// tests if 'ch' is a octal digit
   {
   switch( ch )
@@ -280,9 +281,6 @@ bool ISO_8859::isodigit( const unsigned char ch ) throw()	// tests if 'ch' is a
   }
 
 
-bool ISO_8859::isprint( const unsigned char ch ) throw() { return !iscntrl( ch ); }
-
-
 const char * ISO_8859::control_name( const unsigned char ch ) throw()  // '^A' --> "^A"
   {
   static char buf[3];
@@ -389,7 +387,15 @@ char ISO_8859::toupper( const unsigned char ch ) throw()
   }
 
 
-int ISO_8859::xtoi( const unsigned char ch ) throw()	// returns value of hex digit 'ch'
+char ISO_8859::xdigit( const int value ) throw()
+  {
+  if( value >= 0 && value <= 9 ) return '0' + value;
+  if( value >= 10 && value <= 15 ) return 'A' + value - 10;
+  return 0;
+  }
+
+
+int ISO_8859::xtoi( const unsigned char ch ) throw()
   {
   switch( ch )
     {
