@@ -18,35 +18,42 @@
 
 namespace ISO_8859 {
 
-// 'seq' contains a escape sequence (the characters following a '\')
-// Returns the corresponding code and, in *lenp, the characters read
-// Returns -1 if error
+// 'seq[i]' begins a escape sequence (the characters following a '\').
+// Returns the corresponding code and, in *lenp, the characters read.
+// Returns -1 if error.
 //
-int escape( const std::string & seq, int *lenp = 0 ) throw();
+int escape( const std::string & seq, const unsigned int i, int *lenp = 0 ) throw();
 
 
-// Replace the escape sequences for special characters; "\n" --> '\n'
+// Replace special characters with escape sequences; '\n' --> "\n"
+//
+void escapize( std::string & s ) throw();
+
+
+// Replace escape sequences with special characters; "\n" --> '\n'
 //
 void deescapize( std::string & s ) throw();
 
 
-// Replace tabs with tab escape sequences; '\t' --> "\t"
-//
-void detabize( std::string & s ) throw();
-
-
 bool isalpha_( const unsigned char ch ) throw();
 bool isalnum_( const unsigned char ch ) throw();
-bool isblank( const unsigned char ch ) throw();
-bool iscntrl( const unsigned char ch ) throw();
+inline bool isblank( const unsigned char ch ) throw()
+  { return ( ch == ' ' || ch == '\t' ); }
+
+inline bool iscntrl( const unsigned char ch ) throw()
+  { return ( ch < 32 || ( ch >= 0x7f && ch < 0xA0 ) ); }
+
 bool isodigit( const unsigned char ch ) throw();  // tests if 'ch' is a octal digit
-bool isprint( const unsigned char ch ) throw();
+
+inline bool isprint( const unsigned char ch ) throw()
+  { return !iscntrl( ch ); }
 
 const char * control_name( const unsigned char ch ) throw();  // '^A' --> "^A"
 int controlize( int code ) throw();	// converts 'A' or 'a' into '^A'
 int decontrolize( int code ) throw();	// converts '^A' or 'a' into 'A'
 char tolower( const unsigned char ch ) throw();
 char toupper( const unsigned char ch ) throw();
+char xdigit( const int value ) throw();		// return hex digit for `value'
 int xtoi( const unsigned char ch ) throw();	// returns value of hex digit 'ch'
 
 } // end namespace ISO_8859
