@@ -31,10 +31,11 @@
 #include "screen.h"
 
 
-namespace Menu_file {
+namespace Menu {
 
 void show( const std::vector< std::string > & name_vector, const int lines,
-           const int columns, const int current, Point & cursor ) throw()
+           const int columns, const int current, Point & cursor,
+           const bool indicator = false ) throw()
   {
   static int top_line = 0;
   const int entries = name_vector.size();
@@ -45,18 +46,24 @@ void show( const std::vector< std::string > & name_vector, const int lines,
 
   for( int line = 0; line < lines; ++line )
     {
-    int offset = 0, i_ini = first_entry + ( line * columns );
+    int i1 = -1, i2 = 0, i_ini = first_entry + ( line * columns );
     std::string s;
     for( int i = i_ini; i < i_ini + columns && i < entries; ++i )
       {
-      if( i == current ) { s += "@i"; offset += 2; }	// highlighted entry
-      s += name_vector[i];
-      if( i == current )
-        { cursor = Point( line, s.size() - 2 ); s += "@i"; offset += 2; }
-      const unsigned int col = ( ( ( i % columns ) + 1 ) * Screen::width() ) / columns + offset;
-      if( s.size() > col - 2 ) s.resize( col - 2 );
+      if( i == current )		// highlighted entry
+        {
+        i1 = s.size();
+        if( indicator ) ++i1;		// skip modified indicator
+        s += name_vector[i];
+        i2 = s.size();
+        cursor = Point( line, std::min( (int)s.size(), Screen::width() - 1 ) );
+        }
+      else s += name_vector[i];
+      const unsigned int col = std::min( Screen::width(), ( ( ( i % columns ) + 1 ) * ( Screen::width() + 2 ) ) / columns );
+      if( (int)col < Screen::width() && s.size() > col - 2 ) s.resize( col - 2 );
       while( s.size() < col ) s += ' ';
       }
+    if( i1 >= 0 ) { s.insert( i2, "@i" ); s.insert( i1, "@i" ); }
     Screen::out_line( s, Screen::height() - lines + line - 1 );
     }
   }
@@ -114,12 +121,12 @@ bool valid_filename( const std::string & name ) throw()
   return ( basename.size() && basename != "." && basename != ".." );
   }
 
-} // end namespace Menu_file
+} // end namespace Menu
 
 
 // Returns handle index, or -1 if aborted.
 //
-int Menu_file::bufhandle_menu( int current, const int abort_key ) throw()
+int Menu::bufhandle_menu( int current, const int abort_key ) throw()
   {
   std::vector< std::string > name_vector;
 
@@ -127,7 +134,10 @@ int Menu_file::bufhandle_menu( int current, const int abort_key ) throw()
   for( int i = 0; i < entries; ++i )
     {
     const int c = bufhandle_vector.circular_index( i );
-    name_vector.push_back( bufhandle_vector[c].buffer().uname() );
+    const Buffer & buffer = bufhandle_vector[c].buffer();
+    std::string tmp( " " ); tmp += buffer.uname();
+    if( buffer.modified() ) tmp[0] = '*';
+    name_vector.push_back( tmp );
     }
 
   if( entries < 2 || entries != (int)name_vector.size() ) return -1;
@@ -138,7 +148,7 @@ int Menu_file::bufhandle_menu( int current, const int abort_key ) throw()
       max_width = name_vector[i].size();
   if( max_width == 0 ) return -1;
 
-  const int columns = std::max( 1, Screen::width() / ( max_width + 2 ) );
+  const int columns = std::max( 1, ( Screen::width() + 2 ) / ( max_width + 2 ) );
   const int _height = ( entries + columns - 1 ) / columns;
 
   const int lines = std::min( 10, _height ), line = Screen::height() - lines - 1;
@@ -151,7 +161,7 @@ int Menu_file::bufhandle_menu( int current, const int abort_key ) throw()
   int key = 0;
   while( key >= 0 )
     {
-    show( name_vector, lines, columns, current, cursor );
+    show( name_vector, lines, columns, current, cursor, true );
     Screen::move_to( Point( line, 0 ) + cursor );
     bool valid_key = false;
     while( !valid_key )
@@ -196,7 +206,7 @@ int Menu_file::bufhandle_menu( int current, const int abort_key ) throw()
 // 'name' contains the filename to be modified (may be empty).
 // Returns false if aborted.
 //
-bool Menu_file::file_menu( const std::string & prompt, std::string & name ) throw()
+bool Menu::file_menu( const std::string & prompt, std::string & name ) throw()
   {
   std::string dirname, basename;
   std::vector< std::string > name_vector;
@@ -229,7 +239,7 @@ bool Menu_file::file_menu( const std::string & prompt, std::string & name ) thro
       max_width = name_vector[i].size();
   if( max_width == 0 ) return true;
 
-  const int columns = std::max( 1, Screen::width() / ( max_width + 2 ) );
+  const int columns = std::max( 1, ( Screen::width() + 2 ) / ( max_width + 2 ) );
   const int _height = ( entries + columns - 1 ) / columns;
 
   const int lines = std::min( 10, _height ), line = Screen::height() - lines - 1;
@@ -291,20 +301,20 @@ bool Menu_file::file_menu( const std::string & prompt, std::string & name ) thro
 // Returns the size of the result string, 0 if none, -1 if aborted.
 // 'name' is cleared if aborted.
 //
-int Menu_file::get_filename( const std::string & prompt, std::string & name,
+int Menu::get_filename( const std::string & prompt, std::string & name,
                              const int abort_key ) throw()
   {
   static std::vector< std::string > file_history;
 
   file_history.push_back( name );
-  int len = Screen::get_string( prompt, file_history, abort_key, false, true );
+  int len = Screen::get_string( prompt, file_history, abort_key, true, true );
   if( len > 0 ) name = file_history.back();
   else name.clear();
   return len;
   }
 
 
-bool Menu_file::is_regular( const std::string & name, const bool quiet ) throw()
+bool Menu::is_regular( const std::string & name, const bool quiet ) throw()
   {
   if( name.size() && name[name.size()-1] != '/' )
     {
@@ -312,13 +322,13 @@ bool Menu_file::is_regular( const std::string & name, const bool quiet ) throw()
     const int i = stat( name.c_str(), &st );
     if( ( i == 0 && S_ISREG( st.st_mode ) ) ||
         ( i == -1 && errno == ENOENT ) ) return true;
-    if( !quiet ) Screen::show_message( "Not a regular file", true, true );
+    if( !quiet ) Screen::show_message( "Not a regular file" );
     }
   return false;
   }
 
 
-bool Menu_file::make_path( const std::string & name ) throw()
+bool Menu::make_path( const std::string & name ) throw()
   {
   std::string dirname, basename;
   split_filename( name, dirname, basename );
@@ -344,7 +354,7 @@ bool Menu_file::make_path( const std::string & name ) throw()
   }
 
 
-const std::string & Menu_file::my_basename( const std::string & name ) throw()
+const std::string & Menu::my_basename( const std::string & name ) throw()
   {
   static std::string basename;
   std::string dirname;
@@ -353,7 +363,7 @@ const std::string & Menu_file::my_basename( const std::string & name ) throw()
   }
 
 
-bool Menu_file::test_slash_filename( std::string & name ) throw()
+bool Menu::test_slash_filename( std::string & name ) throw()
   {
   tilde_expansion( name );
   return ( !name.size() ||
@@ -361,7 +371,7 @@ bool Menu_file::test_slash_filename( std::string & name ) throw()
   }
 
 
-bool Menu_file::tilde_expansion( std::string & name ) throw()
+bool Menu::tilde_expansion( std::string & name ) throw()
   {
   if( name.size() >= 2 && !name.compare( 0, 2, "~/" ) )
     { name.replace( 0, 1, RC::home_directory() ); return true; }
