@@ -24,10 +24,10 @@
 #include "screen.h"
 
 
-namespace Menu_help {
+namespace Menu {
 
 const char *help_line[] = {
-  "@i Help Screen: Keyboard Commands               PAGE_DN next         F1 exit help",
+  "@i Help Screen: Keyboard Commands               PAGE_DN next    :    F1 exit help",
   " @bF1@b Help             @bF5@b Prev window     @bF9@b Copy block   ^C  Close window",
   " @bF2@b Save file        @bF6@b Next window    @bF10@b Options      ^X  Save and close",
   " @bF3@b Load file        @bF7@b Undo           @bF11@b Buffers      ^QX Save all and exit",
@@ -37,22 +37,22 @@ const char *help_line[] = {
   " ^[U Beg of file     ^SPACE Mark       ^F Find          Ins Insert/Overwrite",
   " ^[V End of file     ^KB Begin         ^G Find next     Tab Filename completion",
   " ^[L Line            ^KK End                            ^B  Reformat paragraph",
-  " ^[C Column          ^KC Copy          @bDELETE@b           ^P  Control char",
-  " ^[B Beg of block    ^KM Move          ^Y Line          ^S  Show status",
-  " ^[K End of block    ^KY Delete                         ^OC Center line",
-  " ^[G Match pair ->   ^KR Read                           ^OR Repaint screen",
-  " ^[H Match pair <-   ^KW Write                          ^OS Split window",
-  " ^[D Center cursor   ^KI Indent                         ^OV Show version",
-  "                     ^KU Unindent                       ^Q@bF2@b Save named buffers",
-  " @bSCROLL@b",
-  " ^[W Up",
-  " ^[Z Down            @bBOOKMARKS@b",
-  " ^[A Left            ^[ 0-9 Goto",
-  " ^[S Right           ^K 0-9 Set",
+  " ^[C Column          ^KC Copy          @bDELETE@b           ^H  Help",
+  " ^[B Beg of block    ^KM Move          ^Y Line          ^P  Control char",
+  " ^[K End of block    ^KY Delete                         ^S  Show status",
+  " ^[F Match pair ->   ^KR Read          @bENCODE@b           ^OC Center line",
+  " ^[G Match pair <-   ^KW Write         ^O1 Base64       ^OR Repaint screen",
+  " ^[D Center cursor   ^KI Indent        ^O3 Rot-13       ^OS Split window",
+  "                     ^KU Unindent      ^O4 Rot-47       ^OV Show version",
+  " @bSCROLL@b                                                 ^Q@bF2@b Save named buffers",
+  " ^[W Up                                @bDECODE@b           ^QS Show global status",
+  " ^[Z Down            @bBOOKMARKS@b         ^O2 Base64",
+  " ^[A Left            ^[ 0-9 Goto       ^O6 Quot-Print",
+  " ^[S Right           ^K 0-9 Set        ^O8 UTF-8",
   " ",
   " ^[X means Alt-X or Esc-X    ^X means Control-X",
   "",
-  "@i Help Screen: Search/Replace                  PAGE_UP prev         F1 exit help",
+  "@i Help Screen: Search/Replace                  PAGE_UP prev    :    F1 exit help",
   " Escape sequences for special characters:",
   "   \\a  BEL (alert)       \\n  LF (newline)           \\\\   '\\' (backslash)",
   "   \\b  BS  (backspace)   \\r  CR (carriage return)   \\ooo  octal code 'ooo'",
@@ -65,6 +65,7 @@ const char *help_line[] = {
   "   \\[..]   matches one in a set          \\c      matches balanced C expression",
   "   \\[^..]  matches one not in a set      \\+<ch>  matches 0 or more of <ch>",
   "   \\w      matches 0 or more whitespace characters",
+  "   \\W      matches the longest possible non-empty sequence of whitespace",
   " Notes",
   "     Ranges of characters may be specified in sets with 'A-Z'",
   "     '-' may be specified in sets by placing it first or last",
@@ -112,25 +113,27 @@ void show_help( int delta ) throw()
   while( line < Screen::height() ) { Screen::out_line( "", line ); ++line; }
   }
 
-} // end namespace Menu_help
+} // end namespace Menu
 
 
-void Menu_help::help_menu( const int abort_key ) throw()
+void Menu::help_menu( const int abort_key1, const int abort_key2 ) throw()
   {
   Screen::save_lines_and_cursor( 0, Screen::height() );
 
-  int key = abort_key + 1, delta = 0;
-  while( key != abort_key )
+  int delta = 0;
+  bool done = false;
+  while( !done )
     {
     show_help( delta );
     Screen::move_to( Point( 0, Screen::width() - 1 ) );
     while( true )
       {
-      key = Screen::wait_kbhit( 1 );		// show clock on line 0 only
-      if( key == 3 ) key = abort_key;		// ^C
-      if( key == abort_key ) break;
-      if( Screen::convert_key( key ) == Screen::key_npage ) { delta = +1; break; }
-      if( Screen::convert_key( key ) == Screen::key_ppage ) { delta = -1; break; }
+      int key = Screen::wait_kbhit( 1 );	// show clock on line 0 only
+      if( key == 3 || key == abort_key1 || key == abort_key2 )	// ^C
+        { done = true; break; }
+      key = Screen::convert_key( key );
+      if( key == Screen::key_npage ) { delta = +1; break; }
+      if( key == Screen::key_ppage ) { delta = -1; break; }
       }
     }
   Screen::restore_lines_and_cursor();
