@@ -21,13 +21,13 @@
 #include <vector>
 
 #include "buffer.h"
-//#include "block.h"
+#include "block.h"
 #include "menu.h"
 #include "rc.h"
 #include "screen.h"
 
 
-namespace Menu_opts {
+namespace Menu {
 
 int current;		// current highlighted entry in menu
 int entries;		// number of entries in menu
@@ -172,17 +172,18 @@ const char * word_wrap_status( const Buffer & buffer ) throw()
 bool word_wrap_change( Buffer & buffer ) throw()
   { buffer.options.word_wrap = !buffer.options.word_wrap; return true; }
 
-/*
+
 const char * rectangle_mode_status( const Buffer & ) throw()
   { return RC::editor_options().rectangle_mode ? "ON" : "OFF"; }
 
 bool rectangle_mode_change( Buffer & ) throw()
   {
   RC::editor_options().rectangle_mode = !RC::editor_options().rectangle_mode;
-  if( Block::valid() ) Screen::repaint( &Block::buffer() );
+  Screen::show_status_lines();
+  if( Block::bufferp() ) Screen::repaint( Block::bufferp() );
   return true;
   }
-*/
+
 
 struct Entry
   {
@@ -207,7 +208,7 @@ Entry table[] = {
 //  { 'T', "Path to backup files ", 0, 0 },
   { 'U', "Auto unmark ", &auto_unmark_status, &auto_unmark_change },
   { 'W', "Word wrap ", &word_wrap_status, &word_wrap_change },
-//  { 'X', "Rectangle mode ", &rectangle_mode_status, &rectangle_mode_change },
+  { 'X', "Rectangle mode ", &rectangle_mode_status, &rectangle_mode_change },
   { 0, 0, 0, 0 } };
 
 
@@ -260,10 +261,10 @@ void show( const int delta, const Buffer & buffer, Point & cursor ) throw()
     }
   }
 
-} // end namespace Menu_opts
+} // end namespace Menu
 
 
-std::string Menu_opts::options_menu( Buffer & buffer, const int abort_key ) throw()
+void Menu::options_menu( Buffer & buffer, const int abort_key ) throw()
   {
   const int lines = _height, line = Screen::height() - lines;
   Point cursor( Screen::height() - 1, Screen::width() - 1 );
@@ -299,16 +300,16 @@ std::string Menu_opts::options_menu( Buffer & buffer, const int abort_key ) thro
       }
     }
   Screen::restore_lines_and_cursor();
-  std::string msg;
   if( valid_command )
     {
     const Entry & entry = table[current];
     if( entry.change && entry.change( buffer ) )
       {
-      msg = entry.str;
+      std::string msg = entry.str;
       if( entry.status )
         { const char * st = entry.status( buffer ); if( st ) msg += st; }
+      if( msg.size() )
+        { Screen::show_status_lines( &buffer ); Screen::show_message( msg ); }
       }
     }
-  return msg;
   }
