@@ -22,12 +22,12 @@ struct Options
   {
   int keep_lines;
   bool auto_unmark, backup, /*beep,*/ exit_ask, ignore_case;
-  bool orphan_extra, /*rectangle_mode,*/ smart_home, search_wrap;
+  bool orphan_extra, rectangle_mode, smart_home, search_wrap;
 
   Options() throw()
     : keep_lines( -1 ),
     auto_unmark( false ), backup( true ), /*beep( false ),*/ exit_ask( false ),
-    ignore_case( false ), orphan_extra( false ), /*rectangle_mode( false ),*/
+    ignore_case( false ), orphan_extra( false ), rectangle_mode( false ),
     smart_home( true ), search_wrap( false ) {}
 
   void reset() throw() { *this = Options(); }
