@@ -56,7 +56,7 @@ Set::Set( const std::string & regex ) throw()
     if( ch1 == ']' ) { ++i; fail = false; break; }
     if( ch1 == '\\' )
       {
-      int len, cht = ISO_8859::escape( std::string( regex, i + 1 ), &len );
+      int len, cht = ISO_8859::escape( regex, i + 1, &len );
       if( cht < 0 ) break;
       ch1 = cht; i += len;
       }
@@ -68,7 +68,7 @@ Set::Set( const std::string & regex ) throw()
       unsigned char ch2 = regex[i];
       if( ch2 == '\\' )
         {
-        int len, cht = ISO_8859::escape( std::string( regex, i + 1 ), &len );
+        int len, cht = ISO_8859::escape( regex, i + 1, &len );
         if( cht < 0 ) break;
         ch2 = cht; i += len;
         }
@@ -130,7 +130,8 @@ bool match_regex( const Buffer & buffer, Point & p,
                 } break;
       case 'c': {
                 Point begin = p1;
-                if( !buffer.set_to_matching_delimiter( p1, true, true ) || !buffer.pseek( p1, 1 ) )
+                if( buffer.set_to_matching_delimiter( p1, true, true ) <= 0 ||
+                    !buffer.pseek( p1, 1 ) )
                   { fail = true; break; }
                 std::string piece;
                 buffer.to_string( begin, p1, piece );
@@ -164,7 +165,7 @@ bool match_regex( const Buffer & buffer, Point & p,
                     if( !fail ) { pieces.insert( pieces.begin() + ps, piece ); i = regex.size(); }
                     break;
                     }
-                  int len, cht = ISO_8859::escape( std::string( regex, i + 1 ), &len );
+                  int len, cht = ISO_8859::escape( regex, i + 1, &len );
                   if( cht < 0 ) { fail = true; break; }
                   ch1 = cht; i += len;
                   }
@@ -178,6 +179,18 @@ bool match_regex( const Buffer & buffer, Point & p,
                   }
                 if( !fail ) { pieces.insert( pieces.begin() + ps, piece ); i = regex.size(); }
                 } break;
+      case 'W': {			// Find longest matching sequence
+                std::string piece;
+                while( true )
+                  {
+                  Point pn = p1;
+                  const int ch2 = buffer.pgetc( pn );
+                  if( ch2 < 0 || !std::isspace( ch2 ) ) break;
+                  p1 = pn; piece += ch2;
+                  }
+                if( piece.size() ) pieces.push_back( piece );
+                else fail = true;
+                } break;
       case 'w': {			// Find shortest matching sequence
                 const int ps = pieces.size();
                 std::string piece;
@@ -190,7 +203,7 @@ bool match_regex( const Buffer & buffer, Point & p,
                 if( !fail ) { pieces.insert( pieces.begin() + ps, piece ); i = regex.size(); }
                 } break;
       default: {
-               int len, cht = ISO_8859::escape( std::string( regex, i ), &len );
+               int len, cht = ISO_8859::escape( regex, i, &len );
                if( cht < 0 ) { fail = true; break; }
                ch1 = cht;
                int ch2 = buffer.pgetc( p1 );
@@ -295,7 +308,7 @@ bool Regex::replace( Buffer & buffer, const Point & p1, Point & p2,
       }
     else
       {
-      int len, cht = ISO_8859::escape( std::string( regex, i ), &len );
+      int len, cht = ISO_8859::escape( regex, i, &len );
       if( cht >= 0 ) { tmp.pputc( p, cht ); i += ( len - 1 ); }
       else fail = true;
       }
@@ -336,7 +349,7 @@ bool Regex::match_filename( const std::string & regex, const std::string & name
                 } break;
       default: if( ch1 == '\\' )
                  {
-                 int len, cht = ISO_8859::escape( std::string( regex, i1 + 1 ), &len );
+                 int len, cht = ISO_8859::escape( regex, i1 + 1, &len );
                  if( cht < 0 ) { fail = true; break; }
                  ch1 = cht; i1 += ( len - 1 );
                  }
