@@ -32,6 +32,7 @@
 #include "rc.h"
 #include "screen.h"
 #include "window.h"
+#include "window_vector.h"
 
 
 namespace Screen {
@@ -50,7 +51,7 @@ int _height, _width;
 int clock_limit;		//   0: disable clock
 				//   1: showing help
 				// > 1: first line used by menu
-Window_vector window_vector;	// Vector of windows shown on screen
+
 std::vector< Lines_and_cursor > lines_and_cursor_vector;
 
 
@@ -127,7 +128,7 @@ void remove_duplicates( std::vector< std::string > & history ) throw()
 
 void alt_key_loop( int key ) throw()
   {
-  Window & w = window_vector.curwin();
+  Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
@@ -139,8 +140,8 @@ void alt_key_loop( int key ) throw()
     case 'B': w.goto_begin_of_block(); break;
     case 'C': w.goto_column(); break;
     case 'D': w.center_cursor(); break;
-    case 'G': w.goto_matching_delimiter(); break;
-    case 'H': w.goto_matching_delimiter( false ); break;
+    case 'F': w.goto_matching_delimiter(); break;
+    case 'G': w.goto_matching_delimiter( false ); break;
     case 'K': w.goto_end_of_block(); break;
     case 'L': w.goto_line(); break;
     case 'S': w.scroll_horizontal( 8 ); break;
@@ -154,7 +155,8 @@ void alt_key_loop( int key ) throw()
 
 void control_k_loop( int key ) throw()
   {
-  Window & w = window_vector.curwin();
+  Window & w = Window_vector::curwin();
+  const Buffer * old_bufferp = Block::bufferp();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
@@ -162,34 +164,41 @@ void control_k_loop( int key ) throw()
   if( std::isdigit( key ) ) w.set_mark( key - '0' );
   else switch( key )
     {
-    case 'B': Block::set_begin( w.buffer(), w.buffer_handle().pointer );
-              window_vector.repaint(); break;
-    case 'C': window_vector.copy_block(); break;
-    case 'I': window_vector.indent_block(); break;
-    case 'K': Block::set_end( w.buffer(), w.buffer_handle().pointer );
-              window_vector.repaint(); break;
-    case 'M': window_vector.move_block(); break;
-    case 'R': window_vector.read_block(); break;
-    case 'U': window_vector.unindent_block(); break;
-    case 'W': window_vector.write_block(); break;
-    case 'Y': window_vector.delete_block(); break;
+    case 'B': Block::set_begin( w.buffer(), w.pointer() );
+              if( old_bufferp ) repaint( old_bufferp );
+              break;
+    case 'C': Window_vector::copy_block(); break;
+    case 'I': Window_vector::indent_block(); break;
+    case 'K': Block::set_end( w.buffer(), w.pointer() );
+              if( old_bufferp ) repaint( old_bufferp );
+              break;
+    case 'M': Window_vector::move_block(); break;
+    case 'R': Window_vector::read_block(); break;
+    case 'U': Window_vector::unindent_block(); break;
+    case 'W': Window_vector::write_block(); break;
+    case 'Y': Window_vector::delete_block(); break;
     }
   }
 
 
 void control_o_loop( int key, const std::string & version_msg ) throw()
   {
-  Window & w = window_vector.curwin();
+  Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
   switch( key )
     {
-    case 'C': window_vector.center_line(); break;
-    case 'R': werase( stdscr ); wrefresh( stdscr );
-              window_vector.repaint(); break;
-    case 'S': window_vector.split_window(); break;
+    case '1': Window_vector::encode_base64(); break;
+    case '2': Window_vector::decode_base64(); break;
+    case '3': Window_vector::encode_rot1347( true ); break;
+    case '4': Window_vector::encode_rot1347( false ); break;
+    case '6': Window_vector::decode_quoted_printable_utf8( true ); break;
+    case '8': Window_vector::decode_quoted_printable_utf8( false ); break;
+    case 'C': Window_vector::center_line(); break;
+    case 'R': werase( stdscr ); wrefresh( stdscr ); repaint(); break;
+    case 'S': Window_vector::split_window(); break;
     case 'V': show_message( version_msg ); break;
     }
   }
@@ -197,16 +206,17 @@ void control_o_loop( int key, const std::string & version_msg ) throw()
 
 void control_q_loop( int key, int & retval ) throw()
   {
-  Window & w = window_vector.curwin();
+  Window & w = Window_vector::curwin();
   w.show_status_line( ISO_8859::control_name( key ) );
   do key = wait_kbhit(); while( key < 0 );
   w.show_status_line();
   key = ISO_8859::decontrolize( key );
   switch( key )
     {
-    case KEY_F(2): window_vector.save_all_named(); break;
-    case 'C': if( window_vector.close_and_exit( true ) == 0 ) retval = 0; break;
-    case 'X': if( window_vector.close_and_exit( false ) == 0 ) retval = 0; break;
+    case KEY_F(2): bufhandle_vector.save_all_named(); break;
+    case 'C': if( Window_vector::close_and_exit( true ) == 0 ) retval = 0; break;
+    case 'S': bufhandle_vector.show_status(); break;
+    case 'X': if( Window_vector::close_and_exit( false ) == 0 ) retval = 0; break;
     }
   }
 
@@ -216,8 +226,11 @@ void user_control_char() throw()
   const std::string prompt = "Ctrl(A-Z) or 0-255 or 0x00-0xFF or 0-0377 or Escape sequence: ";
   int key;
   while( true )
-    { key = show_message( prompt, true, true );
-      if( key == KEY_F(1) ) Menu_help::help_menu( key ); else break; }
+    {
+    key = show_message( prompt, true, true );
+    if( key == 8 || key == KEY_F(1) ) Menu::help_menu( 8, KEY_F(1) );
+    else break;
+    }
   if( key < 0 || key >= 256 || key == 3 ) return;
   if( key != '\\' && !std::isdigit( key ) )
     { key = ISO_8859::controlize( key ); if( key >= 32 ) key = -1; }
@@ -228,11 +241,11 @@ void user_control_char() throw()
     dummy.back() += key;
     if( get_string( prompt, dummy ) <= 0 ) return;
     if( dummy.back()[0] == '\\' )
-      key = ISO_8859::escape( std::string( dummy.back(), 1 ) );
+      key = ISO_8859::escape( dummy.back(), 1 );
     else if( !RC::parse_int( dummy.back(), key ) ) key = -1;
     }
   if( key < 0 || key >= 256 ) show_message( "Invalid code" );
-  else window_vector.add_char( key, true );
+  else Window_vector::add_char( key, true );
   }
 
 
@@ -274,7 +287,7 @@ bool Screen::init() throw()
   getmaxyx( stdscr, _height, _width );
   if( _height < 24 || _width < 80 ) return false;
   clock_limit = 0;
-  window_vector.init();
+  Window_vector::init();
   return true;
   }
 
@@ -296,9 +309,9 @@ void Screen::clock_handler( const int sig ) throw()	// updates the clock
     Point old_cursor; save_cursor( old_cursor );
     if( clock_limit == 1 )
       out_raw_string( 0, 60, s, 5, 0, A_REVERSE );
-    else for( int i = 0; i < window_vector.windows(); ++i )
+    else for( int i = 0; i < Window_vector::windows(); ++i )
       {
-      Point cursor = window_vector[i].clock_position();
+      Point cursor = Window_vector::window( i ).clock_position();
       if( cursor.line >= clock_limit ) break;
       out_raw_string( cursor.line, cursor.col, s, 5, 0, A_REVERSE );
       }
@@ -322,7 +335,7 @@ int Screen::wait_kbhit( const int new_clock_limit, const bool get_key,
   if( force_refresh || kbhit() < 0 )
     {
     if( new_clock_limit < 0 )
-      move_to( window_vector.curwin().absolute_cursor() );
+      move_to( Window_vector::curwin().absolute_cursor() );
 
     clock_limit = ( new_clock_limit >= 0 ) ? new_clock_limit : _height;
     if( clock_limit > 0 ) { clock_handler( 0 ); alarm( 1 ); }
@@ -364,27 +377,35 @@ int Screen::get_string( const std::string & prompt,
     if( key == abort_key ) key = 3;
     switch( key )
       {
-      case   3 : key = -1; break;		// ^C
+      case   3 : key = -1; break;				// ^C
       case '\t': if( usetab )
                    {
                    std::string name( s, prompt.size(), s.size() );
-                   if( !Menu_file::file_menu( prompt, name ) ) { key = -1; break; }
+                   if( !Menu::file_menu( prompt, name ) ) { key = -1; break; }
                    s.erase( prompt.size(), s.size() ); s += name;
                    scol = s.size(); modified = true;
                    }
                  break;
-      case '\r': { std::string name( s, prompt.size(), s.size() );
-                   if( !usetab || !name.size() ||
-                       ( Menu_file::test_slash_filename( name ) && Menu_file::is_regular( name ) ) )
-                     { key = -2; history.back() = name; } }
+      case '\r': {
+                 std::string name( s, prompt.size(), s.size() );
+                 if( !usetab || !name.size() )
+                   { key = -2; history.back() = name; }
+                 else if( Menu::test_slash_filename( name ) )
+                   { if( !Menu::is_regular( name ) ) key = -1;
+                     else { key = -2; history.back() = name; } }
+                 }
                  break;
-      case KEY_F(1) : Menu_help::help_menu( key ); break;
-      case KEY_F(9) : if( allow_copy && Block::valid() &&
-                          Block::begin().line == Block::end().line )
+      case  25 : if( s.size() > prompt.size() )			// ^Y
+                   { s.resize( prompt.size() ); scol = s.size();
+                     modified = true; }
+                 break;
+      case   8 :						// ^H
+      case KEY_F(1) : Menu::help_menu( 8, KEY_F(1) ); break;
+      case KEY_F(9) : if( allow_copy && Block::valid() )
                         {
                         std::string tmp;
-                        Block::buffer().to_string( Block::begin(), Block::end(), tmp );
-                        ISO_8859::detabize( tmp );
+                        Block::bufferp()->to_string( Block::begin(), Block::end(), tmp );
+                        ISO_8859::escapize( tmp );
                         if( tmp.size() )
                           { s.insert( scol, tmp ); scol += tmp.size(); modified = true; }
                         }
@@ -429,7 +450,7 @@ int Screen::get_string( const std::string & prompt,
   if( original.size() && original != history.back() )
     history.insert( history.end() - 1, original );
   if( key != -1 ) remove_duplicates( history );
-  if( ( key == -1 && !original.size() ) || history.back().size() == 0 )
+  if( ( key == -1 && !original.size() ) || !history.back().size() )
     { history.pop_back(); if( key != -1 ) return 0; }
   if( key == -1 ) return -1;
   return history.back().size();
@@ -520,6 +541,20 @@ bool Screen::out_line( const std::string & s, const int line, const bool shift,
   }
 
 
+void Screen::repaint( const Buffer * bufp ) throw()
+  {
+  for( int i = 0; i < Window_vector::windows(); ++i )
+    {
+    Window & w = Window_vector::window( i );
+    if( !bufp || bufp == &w.buffer() )
+      {
+      if( bufp == &w.buffer() ) w.update_points( w.pointer(), false );
+      w.repaint();
+      }
+    }
+  }
+
+
 int Screen::show_message( const std::string & s, const bool get_key,
                           const bool take_cursor ) throw()
   {
@@ -537,6 +572,14 @@ int Screen::show_message( const std::string & s, const bool get_key,
   }
 
 
+void Screen::show_status_lines( const Buffer * bufp ) throw()
+  {
+  for( int i = 0; i < Window_vector::windows(); ++i )
+    if( !bufp || bufp == &Window_vector::window( i ).buffer() )
+      Window_vector::window( i ).show_status_line();
+  }
+
+
 Screen::Keys Screen::convert_key( const int key ) throw()
   {
   switch( key )
@@ -561,41 +604,41 @@ int Screen::user_loop( const std::string & version_msg ) throw()
   bool force_refresh = false;
   while( retval < 0 )
     {
-    int key;
-    do key = wait_kbhit( -1, true, force_refresh ); while( key < 0 );
+    const int key = wait_kbhit( -1, true, force_refresh );
+    if( key < 0 ) continue;
     force_refresh = false;
-    Window & w = window_vector.curwin();
+    Window & w = Window_vector::curwin();
     switch( key )
       {
-      case  0: window_vector.toggle_marking(); break;		// ^SPACE
-      case  2: window_vector.reformat(); break;			// ^B
-      case  3: if( !window_vector.close( true ) ) retval = 0; break;	// ^C
-      case  6: window_vector.search(); break;			// ^F
-      case  7: window_vector.search( true ); break;		// ^G
+      case  0: Block::toggle_marking( w.buffer(), w.pointer() ); break;	// ^SPACE
+      case  2: Window_vector::reformat(); break;		// ^B
+      case  3: if( !Window_vector::close( true ) ) retval = 0; break;	// ^C
+      case  6: Window_vector::search( key ); break;		// ^F
+      case  7: Window_vector::search( key, true ); break;	// ^G
       case 11: control_k_loop( key ); break;			// ^K
-      case 12: window_vector.search( true ); break;		// ^L
       case 15: control_o_loop( key, version_msg ); break;	// ^O
       case 16: user_control_char(); break;			// ^P
       case 17: control_q_loop( key, retval ); break;		// ^Q
       case 19: w.show_character_info(); break;			// ^S
-      case 24: if( !window_vector.close( false ) ) retval = 0; break;	// ^X
-      case 25: window_vector.delete_line(); break;		// ^Y
+      case 24: if( !Window_vector::close( false ) ) retval = 0; break;	// ^X
+      case 25: Window_vector::delete_line(); break;		// ^Y
       case 27: alt_key_loop( key ); break;			// Alt or Esc
-      case KEY_F(1) : Menu_help::help_menu( key ); break;
-      case KEY_F(2) : window_vector.save_file( key ); break;
-      case KEY_F(3) : window_vector.load_file( key ); break;
-      case KEY_F(4) : window_vector.load( -1 ); break;
-      case KEY_F(5) : window_vector.prev(); break;
-      case KEY_F(6) : window_vector.next(); break;
-      case KEY_F(7) : window_vector.undo(); break;
-      case KEY_F(8) : window_vector.redo(); break;
-      case KEY_F(9) : window_vector.copy_block(); break;
-      case KEY_F(10): window_vector.options_menu( w.buffer(), key ); break;
-      case KEY_F(11): window_vector.bufhandle_menu( key ); break;
-      case KEY_BACKSPACE: window_vector.delete_char( true ); break;
-      case KEY_DC   : window_vector.delete_char(); break;
+      case  8:							// ^H
+      case KEY_F(1) : Menu::help_menu( 8, KEY_F(1) ); break;
+      case KEY_F(2) : Window_vector::save_file( key ); break;
+      case KEY_F(3) : Window_vector::load_file( key ); break;
+      case KEY_F(4) : if( !Window_vector::load( -1 ) ) retval = 3; break;
+      case KEY_F(5) : Window_vector::prev(); break;
+      case KEY_F(6) : Window_vector::next(); break;
+      case KEY_F(7) : Window_vector::undo(); break;
+      case KEY_F(8) : Window_vector::redo(); break;
+      case KEY_F(9) : Window_vector::copy_block(); break;
+      case KEY_F(10): Menu::options_menu( w.buffer(), key ); break;
+      case KEY_F(11): Window_vector::bufhandle_menu( key ); break;
+      case KEY_BACKSPACE: Window_vector::delete_char( true ); break;
+      case KEY_DC   : Window_vector::delete_char(); break;
       case KEY_IC   : w.buffer().options.overwrite = !w.buffer().options.overwrite;
-                      window_vector.show_status_lines( &w.buffer() ); break;
+                      show_status_lines( &w.buffer() ); break;
       case KEY_HOME : w.goto_home(); break;
       case KEY_END  : w.goto_eol(); break;
       case KEY_PPAGE: w.move_page( false ); break;
@@ -606,10 +649,10 @@ int Screen::user_loop( const std::string & version_msg ) throw()
       case KEY_RIGHT: w.goto_pnext(); break;
       default       : if( key < 256 )
                         { if( key == '\r' || key == '\n' ) force_refresh = true;
-                          window_vector.add_char( key ); }
+                          Window_vector::add_char( key ); }
       }
-    if( Block::follow_marking( w.buffer(), w.buffer_handle().pointer ) )
-      window_vector.repaint( &w.buffer() );
+    if( Block::follow_marking( w.buffer(), w.pointer() ) )
+      repaint( &w.buffer() );
     }
   wmove( stdscr, _height - 1, 0 );	// clear bottom line on exit
   wclrtoeol( stdscr ); wrefresh( stdscr );
