@@ -59,8 +59,10 @@ void restore_lines_and_cursor() throw();
 void out_buffer_line( const Buffer & buffer, Point c, const int line ) throw();
 bool out_line( const std::string & s, const int line, const bool shift = false,
                const char mode = 0 ) throw();
+void repaint( const Buffer * bufp = 0 ) throw();
 int show_message( const std::string & s, const bool get_key = false,
                   const bool take_cursor = false ) throw();
+void show_status_lines( const Buffer * bufp = 0 ) throw();
 
 Keys convert_key( const int key ) throw();
 int user_loop( const std::string & version_msg ) throw();
