@@ -23,15 +23,21 @@
 
 #include "buffer.h"
 #include "block.h"
+#include "encoding.h"
 #include "iso_8859.h"
 #include "menu.h"
 #include "rc.h"
 #include "regex.h"
 #include "screen.h"
 #include "window.h"
+#include "window_vector.h"
 
 
-namespace {
+namespace Window_vector {
+
+int _curwin;			// current window (the one with the cursor in)
+std::vector< Window > data;
+
 
 bool no_block() throw()
   {
@@ -42,12 +48,23 @@ bool no_block() throw()
 
 bool no_block_in_this_file( const Buffer & buffer ) throw()
   {
-  if( !Block::same_buffer( buffer ) )
+  if( Block::bufferp() != &buffer )
     { Screen::show_message( "Block is not in this file" ); return true; }
   return false;
   }
 
 
+bool rectangular_block() throw()
+  {
+  if( RC::editor_options().rectangle_mode )
+    {
+    Screen::show_message( "Operation not available in rectangular block mode" );
+    return true;
+    }
+  return false;
+  }
+
+
 bool read_only( const Buffer & buffer ) throw()
   {
   if( buffer.options.read_only )
@@ -56,14 +73,18 @@ bool read_only( const Buffer & buffer ) throw()
   }
 
 
-void update_points( std::vector< Window > & data, const Buffer * bufp ) throw()
+void replace_and_repaint( Buffer & buffer, const Basic_buffer & tmp ) throw()
   {
-  for( unsigned int i = 0; i < data.size(); ++i )
-    if( !bufp || bufp == &data[i].buffer() )
-      data[i].update_points( data[i].buffer_handle().pointer, false );
+  Point p = Block::end();
+  buffer.reset_appendable();
+  buffer.replace( Block::begin(), p, tmp, tmp.bof(), tmp.eof() );
+  buffer.reset_appendable();
+  if( RC::editor_options().auto_unmark ) Block::reset();
+  else Block::set_end( buffer, p );
+  Screen::repaint( &buffer );
   }
 
-} // end namespace
+} // end namespace Window_vector
 
 
 int Window_vector::init() throw()
@@ -94,7 +115,7 @@ int Window_vector::load( int new_size ) throw()
   new_size = std::min( new_size, Screen::height() / Window::min_height );
   new_size = std::max( new_size, 1 );
   while( _curwin >= new_size ) { --_curwin; bufhandle_vector.next(); }
-  int wheight = Screen::height() / new_size;
+  const int wheight = Screen::height() / new_size;
 
   data.clear();
   int i = 0, y = 0;
@@ -112,7 +133,7 @@ int Window_vector::load( int new_size ) throw()
 
 void Window_vector::bufhandle_menu( const int abort_key ) throw()
   {
-  const int handle = Menu_file::bufhandle_menu( _curwin, abort_key );
+  const int handle = Menu::bufhandle_menu( _curwin, abort_key );
   if( handle >= 0 ) { _curwin = handle; load(); }
   }
 
@@ -120,7 +141,7 @@ void Window_vector::bufhandle_menu( const int abort_key ) throw()
 void Window_vector::load_file( const int abort_key ) throw()
   {
   std::string name;
-  if( Menu_file::get_filename( "Name of file to edit (^C to abort): ", name, abort_key ) < 0 )
+  if( Menu::get_filename( "Name of file to edit (^C to abort): ", name, abort_key ) < 0 )
     return;
   const int old_handles = bufhandle_vector.handles();
   int handle;
@@ -141,32 +162,23 @@ void Window_vector::split_window() throw()
   }
 
 
-void Window_vector::save_file( const int abort_key ) const throw()
+void Window_vector::save_file( const int abort_key ) throw()
   {
   const Buffer & buffer = data[_curwin].buffer();
 
   std::string name = buffer.name();
-  if( Menu_file::get_filename( "Name of file to save (^C to abort): ", name, abort_key ) <= 0 )
+  if( Menu::get_filename( "Name of file to save (^C to abort): ", name, abort_key ) <= 0 )
     return;
   bool done;
   try { done = buffer.save( &name ); }
   catch( Buffer::Error e ) { Screen::show_message( e.s ); return; }
   if( done )
     {
-    show_status_lines( &buffer );
+    Screen::show_status_lines( &buffer );
     std::string s( "File '" ); s += name; s += "' saved";
     Screen::show_message( s );
     }
-  else Screen::show_message( "File not saved" );
-  }
-
-
-void Window_vector::save_all_named() const throw()
-  {
-  const bool done = bufhandle_vector.save_all_named();
-  show_status_lines();
-  if( done ) Screen::show_message( "All named buffers saved" );
-  else Screen::show_message( "Not all named buffers saved" );
+  else if( !read_only( buffer ) ) Screen::show_message( "File not saved" );
   }
 
 
@@ -192,7 +204,7 @@ int Window_vector::close( bool abort, const bool quiet ) throw()
       {
       std::string name = bh.buffer().name();
       if( exit_ask || !name.size() )
-        while( !Menu_file::get_filename( "Name of file to save (^C to abort): ", name ) );
+        while( !Menu::get_filename( "Name of file to save (^C to abort): ", name ) );
       if( !name.size() ) return -1;
       try { if( !bh.buffer().save( &name ) ) return -1; }
       catch( Buffer::Error e )
@@ -225,6 +237,9 @@ int Window_vector::close_and_exit( const bool abort ) throw()
   }
 
 
+Window & Window_vector::curwin() throw() { return data[_curwin]; }
+
+
 void Window_vector::next() throw()
   {
   const int handles = bufhandle_vector.handles();
@@ -251,13 +266,16 @@ void Window_vector::prev() throw()
   }
 
 
-Window & Window_vector::operator[]( int i ) throw()
+Window & Window_vector::window( int i ) throw()
   {
   if( i < 0 ) i = 0; else if( i >= windows() ) i = windows() - 1;
   return data[i];
   }
 
 
+int Window_vector::windows() throw() { return data.size(); }
+
+
 void Window_vector::add_char( unsigned char ch, const bool force ) throw()
   {
   Window & w = data[_curwin];
@@ -265,18 +283,18 @@ void Window_vector::add_char( unsigned char ch, const bool force ) throw()
 
   if( ch == '\r' && !force ) ch = '\n';
   if( ( force || ch == '\t' || ch == '\n' || ISO_8859::isprint( ch ) ) &&
-      w.buffer().pputc( w.buffer_handle().pointer, ch ) )
-    { update_points( data, &w.buffer() ); repaint( &w.buffer() ); }
+      w.buffer().pputc( w.pointer(), ch ) )
+    Screen::repaint( &w.buffer() );
   }
 
 
 void Window_vector::delete_char( const bool back ) throw()
   {
   Window & w = data[_curwin];
-  Point & p = w.buffer_handle().pointer;
+  Point & p = w.pointer();
 
   if( !read_only( w.buffer() ) && w.buffer().pdelc( p, back ) )
-    { update_points( data, &w.buffer() ); repaint( &w.buffer() ); }
+    Screen::repaint( &w.buffer() );
   }
 
 
@@ -284,8 +302,24 @@ void Window_vector::delete_line() throw()
   {
   Window & w = data[_curwin];
 
-  if( !read_only( w.buffer() ) && w.buffer().pdell( w.buffer_handle().pointer ) )
-    { update_points( data, &w.buffer() ); repaint( &w.buffer() ); }
+  if( !read_only( w.buffer() ) && w.buffer().pdell( w.pointer() ) )
+    Screen::repaint( &w.buffer() );
+  }
+
+
+void Window_vector::copy_block() throw()
+  {
+  Window & w = data[_curwin];
+
+  if( !no_block() && !read_only( w.buffer() ) )
+    {
+    const Buffer & buffer = *Block::bufferp();
+    if( Block::copy_block( w.buffer(), w.pointer() ) )
+      {
+      if( &buffer != &w.buffer() ) Screen::repaint( &buffer );
+      Screen::repaint( &w.buffer() );
+      }
+    }
   }
 
 
@@ -295,34 +329,24 @@ void Window_vector::delete_block() throw()
 
   if( !no_block() && !no_block_in_this_file( buffer ) &&
       !read_only( buffer ) && Block::delete_block() )
-    { update_points( data, &buffer ); repaint( &buffer ); }
-  }
-
-
-void Window_vector::copy_block() throw()
-  {
-  Window & w = data[_curwin];
-
-  if( !no_block() && !read_only( w.buffer() ) &&
-      Block::copy_block( w.buffer(), w.buffer_handle().pointer ) )
-    { update_points( data, &w.buffer() ); repaint(); }
+    Screen::repaint( &buffer );
   }
 
 
 void Window_vector::move_block() throw()
   {
   Window & w = data[_curwin];
-  Point & p = w.buffer_handle().pointer;
+  Point & p = w.pointer();
 
-  if( no_block() || Block::in_block_or_end( w.buffer(), p ) ) return;
-  const Buffer & buffer = Block::buffer();
-  if( !read_only( w.buffer() ) && !read_only( buffer ) &&
-      Block::move_block( w.buffer(), p ) )
+  if( !no_block() && !Block::in_block_or_end( w.buffer(), p ) )
     {
-    for( unsigned int i = 0; i < data.size(); ++i )
-      if( &w.buffer() == &data[i].buffer() || &buffer == &data[i].buffer() )
-        data[i].update_points( data[i].buffer_handle().pointer, false );
-    repaint();
+    const Buffer & buffer = *Block::bufferp();
+    if( !read_only( w.buffer() ) && !read_only( buffer ) &&
+        Block::copy_block( w.buffer(), p, true ) )
+      {
+      if( &buffer != &w.buffer() ) Screen::repaint( &buffer );
+      Screen::repaint( &w.buffer() );
+      }
     }
   }
 
@@ -333,27 +357,43 @@ void Window_vector::read_block() throw()
   if( read_only( w.buffer() ) ) return;
 
   std::string name;
-  if( Menu_file::get_filename( "Name of file to insert (^C to abort): ", name ) <= 0 )
+  if( Menu::get_filename( "Name of file to insert (^C to abort): ", name ) <= 0 )
     return;
   try {
     Buffer tmp( RC::default_buffer_options(), &name );
     if( tmp.empty() ) throw Buffer::Error( "File not found" );
-    Point p = w.buffer_handle().pointer;
+    Point p = w.pointer();
     if( w.buffer().pputb( p, tmp, tmp.bof(), tmp.eof() ) )
-      { update_points( data, &w.buffer() ); repaint(); }
+      Screen::repaint( &w.buffer() );
     }
   catch( Buffer::Error e ) { Screen::show_message( e.s ); }
   }
 
 
-void Window_vector::write_block() const throw()
+void Window_vector::write_block() throw()
   {
   if( no_block() ) return;
   std::string name;
-  if( Menu_file::get_filename( "Name of file to write (^C to abort): ", name ) <= 0 )
+  if( Menu::get_filename( "Name of file to write (^C to abort): ", name ) <= 0 )
     return;
-  try { Block::buffer().save( &name, false, false,
-                              Block::begin(), Block::end() ); }
+  try {
+    if( !RC::editor_options().rectangle_mode )
+      Block::bufferp()->save( &name, false, false, Block::begin(), Block::end() );
+    else
+      {
+      Buffer::Options options;
+      Buffer tmp( options );
+      Block::save_block_position();
+      Block::copy_block( tmp, tmp.bof() );
+      Block::restore_block_position();
+      tmp.save( &name );
+      }
+    if( RC::editor_options().auto_unmark )
+      {
+      const Buffer * bufferp = Block::bufferp();
+      Block::reset(); Screen::repaint( bufferp );
+      }
+    }
   catch( Buffer::Error e ) { Screen::show_message( e.s ); }
   }
 
@@ -362,8 +402,8 @@ void Window_vector::indent_block() throw()
   {
   Buffer & buffer = data[_curwin].buffer();
 
-  if( no_block() || no_block_in_this_file( buffer ) || read_only( buffer ) )
-    return;
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return;
   Point p = Block::begin();
   bool first_post = true;
   const bool ow = buffer.options.overwrite;
@@ -381,7 +421,8 @@ void Window_vector::indent_block() throw()
     }
   buffer.reset_appendable();
   buffer.options.overwrite = ow;
-  update_points( data, &buffer ); repaint( &buffer );
+  if( RC::editor_options().auto_unmark ) Block::reset();
+  Screen::repaint( &buffer );
   }
 
 
@@ -389,8 +430,8 @@ void Window_vector::unindent_block() throw()
   {
   Buffer & buffer = data[_curwin].buffer();
 
-  if( no_block() || no_block_in_this_file( buffer ) || read_only( buffer ) )
-    return;
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return;
   Point p = Block::begin();
   bool fail = false;
   while( buffer.bol( p ) < Block::end() && !fail )
@@ -430,7 +471,8 @@ void Window_vector::unindent_block() throw()
     }
   buffer.reset_appendable();
   buffer.options.overwrite = ow;
-  update_points( data, &buffer ); repaint( &buffer );
+  if( RC::editor_options().auto_unmark ) Block::reset();
+  Screen::repaint( &buffer );
   }
 
 
@@ -440,8 +482,7 @@ void Window_vector::undo() throw()
 
   if( !read_only( w.buffer() ) )
     {
-    if( w.buffer().undo( w.buffer_handle().pointer ) )
-      { update_points( data, &w.buffer() ); repaint( &w.buffer() ); }
+    if( w.buffer().undo( w.pointer() ) ) Screen::repaint( &w.buffer() );
     else Screen::show_message( "Nothing to undo" );
     }
   }
@@ -453,8 +494,7 @@ void Window_vector::redo() throw()
 
   if( !read_only( w.buffer() ) )
     {
-    if( w.buffer().redo( w.buffer_handle().pointer ) )
-      { update_points( data, &w.buffer() ); repaint( &w.buffer() ); }
+    if( w.buffer().redo( w.pointer() ) ) Screen::repaint( &w.buffer() );
     else Screen::show_message( "Nothing to redo" );
     }
   }
@@ -464,7 +504,7 @@ void Window_vector::center_line() throw()
   {
   Window & w = data[_curwin];
   Buffer & buffer = w.buffer();
-  const Point bol = buffer.bol( w.buffer_handle().pointer );
+  const Point bol = buffer.bol( w.pointer() );
   const Point bot = buffer.bot( bol );
   const Point cbot = buffer.to_cursor( bot );
   const Point ceot = buffer.to_cursor( buffer.eot( bol ) );
@@ -483,7 +523,7 @@ void Window_vector::center_line() throw()
       }
     else buffer.pdelb( bol, Point( bot.line, -dif ) );
     buffer.reset_appendable();
-    update_points( data, &buffer ); repaint( &buffer );
+    Screen::repaint( &buffer );
     }
   }
 
@@ -492,8 +532,8 @@ void Window_vector::reformat() throw()
   {
   Window & w = data[_curwin];
   Buffer & buffer = w.buffer();
-  const Point bop = buffer.bop( w.buffer_handle().pointer );
-  const Point eop = buffer.eop( w.buffer_handle().pointer );
+  const Point bop = buffer.bop( w.pointer() );
+  const Point eop = buffer.eop( w.pointer() );
   const Point bot = buffer.bot( bop );
 
   if( !read_only( buffer ) && bop < eop )
@@ -526,49 +566,97 @@ void Window_vector::reformat() throw()
     if( change )
       {
       buffer.reset_appendable();
-      buffer.pdelb( bop, eop );
-      buffer.force_append();
-      Point p = bop; buffer.pputb( p, tmp, tmp.bof(), tmp.eof() );
+      Point p = eop;
+      buffer.replace( bop, p, tmp, tmp.bof(), tmp.eof() );
       buffer.reset_appendable();
-      update_points( data, &buffer ); repaint( &buffer );
+      Screen::repaint( &buffer );
       }
     }
   }
 
 
-void Window_vector::options_menu( Buffer & buffer, const int abort_key ) const throw()
+void Window_vector::encode_base64() throw()
   {
-  const std::string msg = Menu_opts::options_menu( buffer, abort_key );
-  if( msg.size() )
-    { show_status_lines( &buffer ); Screen::show_message( msg ); }
+  Buffer & buffer = data[_curwin].buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return;
+  std::string in, out;
+  Block::bufferp()->to_string( Block::begin(), Block::end(), in );
+  Encoding::base64_encode( in, out );
+  Basic_buffer tmp;
+  Point p = tmp.bof();
+  int c = 0;
+  for( unsigned int i = 0; i < out.size(); ++i )
+    {
+    tmp.pputc( p, out[i] );
+    if( ++c >= 72 ) { c = 0; tmp.pputc( p, '\n' ); }
+    }
+  if( c ) tmp.pputc( p, '\n' );
+  replace_and_repaint( buffer, tmp );
   }
 
 
-void Window_vector::toggle_marking() const throw()
+void Window_vector::decode_base64() throw()
   {
-  const Window & w = data[_curwin];
-  const char * msg = Block::toggle_marking( w.buffer(), w.buffer_handle().pointer );
-  if( msg ) { repaint(); Screen::show_message( msg ); }
+  Buffer & buffer = data[_curwin].buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return;
+  std::string in, out;
+  Block::bufferp()->to_string( Block::begin(), Block::end(), in );
+  for( int i = in.size() - 1; i >= 0; --i )
+    if( std::isspace( in[i] ) )
+      in.erase( i, 1 );
+  if( !Encoding::base64_decode( in, out ) )
+    { Screen::show_message( "Invalid Base64 data in block" ); return; }
+  Basic_buffer tmp;
+  Point p = tmp.bof();
+  for( unsigned int i = 0; i < out.size(); ++i )
+    tmp.pputc( p, out[i] );
+  replace_and_repaint( buffer, tmp );
   }
 
 
-void Window_vector::repaint( const Buffer * bufp ) const throw()
+void Window_vector::decode_quoted_printable_utf8( const bool mode_q ) throw()
   {
-  for( int i = 0; i < windows(); ++i )
-    if( !bufp || bufp == &data[i].buffer() )
-      data[i].repaint();
+  Buffer & buffer = data[_curwin].buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return;
+  std::string in, out;
+  Block::bufferp()->to_string( Block::begin(), Block::end(), in );
+  if( mode_q )
+    { if( !Encoding::quoted_printable_decode( in, out ) )
+      { Screen::show_message( "Invalid Quoted-Printable data in block" ); return; } }
+  else
+    { if( !Encoding::utf8_decode( in, out ) )
+      { Screen::show_message( "Invalid UTF-8 data in block" ); return; } }
+  if( in.size() == out.size() )
+    { Screen::show_message( "Text in block is already decoded" ); return; }
+  Basic_buffer tmp;
+  Point p = tmp.bof();
+  for( unsigned int i = 0; i < out.size(); ++i )
+    tmp.pputc( p, out[i] );
+  replace_and_repaint( buffer, tmp );
   }
 
 
-void Window_vector::show_status_lines( const Buffer * bufp, const char * prefix ) const throw()
+void Window_vector::encode_rot1347( const bool mode13 ) throw()
   {
-  for( int i = 0; i < windows(); ++i )
-    if( !bufp || bufp == &data[i].buffer() )
-      data[i].show_status_line( prefix );
+  Buffer & buffer = data[_curwin].buffer();
+  if( no_block() || no_block_in_this_file( buffer ) || rectangular_block() ||
+      read_only( buffer ) ) return;
+  Basic_buffer tmp;
+  Point p = tmp.bof();
+  if( mode13 )
+    for( Point p1 = Block::begin(); p1 < Block::end(); )
+      tmp.pputc( p, Encoding::rot13( buffer.pgetc( p1 ) ) );
+  else
+    for( Point p1 = Block::begin(); p1 < Block::end(); )
+      tmp.pputc( p, Encoding::rot47( buffer.pgetc( p1 ) ) );
+  replace_and_repaint( buffer, tmp );
   }
 
 
-void Window_vector::search( const bool again ) throw()
+void Window_vector::search( const int abort_key, const bool again ) throw()
   {
   static std::vector< std::string > search_history;
   static std::vector< std::string > replace_history;
@@ -581,7 +669,7 @@ void Window_vector::search( const bool again ) throw()
   if( !again || search_history.size() == 0 )
     {
     search_history.push_back( std::string() );
-    if( Screen::get_string( "Find (^C to abort): ", search_history, 3, true ) <= 0 )
+    if( Screen::get_string( "Find (^C to abort): ", search_history, abort_key, true ) <= 0 )
       return;
     search_string = search_history.back(); ISO_8859::deescapize( search_string );
     std::string prompt;
@@ -589,7 +677,7 @@ void Window_vector::search( const bool again ) throw()
       prompt = "(N)o_ignore (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
     else prompt = "(I)gnore (R)eplace (B)ackwards (G)lobal Bloc(K) (^C to abort): ";
     options_history.push_back( std::string() );
-    const int result = Screen::get_string( prompt, options_history );
+    const int result = Screen::get_string( prompt, options_history, abort_key );
     if( result < 0 ) return;
     backward = false; block = false; global = false; replace = false;
     icase = RC::editor_options().ignore_case;
@@ -611,7 +699,7 @@ void Window_vector::search( const bool again ) throw()
       {
       replace_string.clear();
       replace_history.push_back( std::string() );
-      const int len = Screen::get_string( "Replace with (^C to abort): ", replace_history, 3, true );
+      const int len = Screen::get_string( "Replace with (^C to abort): ", replace_history, abort_key, true );
       if( len < 0 ) { replace = false; return; }
       if( len > 0 )
         { replace_string = replace_history.back();
@@ -619,12 +707,10 @@ void Window_vector::search( const bool again ) throw()
       }
     }
 
-  Point p = curwin().buffer_handle().pointer;
+  Point p = curwin().pointer();
   if( block )
     {
     if( no_block() || no_block_in_this_file( curwin().buffer() ) ) return;
-//    if( RC::editor_options().rectangle_mode )
-//      { Screen::show_message( "Can't search block in rectangle mode" ); return; }
     if( !again || !Block::in_block_or_end( curwin().buffer(), p ) )
       { if( backward ) p = Block::end(); else p = Block::begin(); }
     }
@@ -636,21 +722,32 @@ void Window_vector::search( const bool again ) throw()
     std::vector< std::string > regex_pieces;
     if( replace && empty_replace && !backward ) w.buffer().pnext( p );
     Point p1 = p, p2 = p;
-    found = Regex::find( w.buffer(), p1, p2, search_string, regex_pieces, icase, backward );
+    found = Regex::find( w.buffer(), p1, p2, search_string, regex_pieces,
+                         icase, backward );
     empty_replace = ( found && ( p1 == p2 ) );
-    if( found && block && ( p1 < Block::begin() || p1 >= Block::end() || p2 > Block::end() ) )
+    if( found && block &&
+        ( p1 < Block::begin() || p1 >= Block::end() || p2 > Block::end() ) )
       found = false;
     if( backward ) p = p1; else p = p2;
 
     if( found )
       {
+      if( block && RC::editor_options().rectangle_mode &&
+          ( p1.col < Block::begin().col || p2.col > Block::end().col ) )
+        { ++i; found = false; continue; }
       found_any = true;
       if( !replace ) w.update_points( p, true, true );
       else
         {
         std::string error;
         Block::save_block_position(); w.update_points( p, false, true );
-        if( ask ) { Block::set_block( w.buffer(), p1, p2 ); w.repaint(); }
+        if( ask )
+          {
+          const bool rm = RC::editor_options().rectangle_mode;
+          RC::editor_options().rectangle_mode = false;
+          Block::set_block( w.buffer(), p1, p2 ); w.repaint();
+          RC::editor_options().rectangle_mode = rm;
+          }
         while( true )
           {
           int key = 0;
